import argparse
import sys

import lxml.etree as etree

from xapiparser import __version__
from xapiparser import parse

parser = argparse.ArgumentParser(description='xAPI ssh expression parser.')
parser.add_argument('expression', metavar='expression', nargs=1,
                    help="xAPI ssh expression")
parser.add_argument('--version', action="store_true",
                    help="version")


def main(args=None):
    args = parser.parse_args(args=args)
    if args.version:
        print(f"version: {__version__}")
        sys.exit()
    print(etree.tostring(parse(args.expression[0]), pretty_print=True, encoding='unicode'))



import re
import shlex

from lxml.etree import Element
from lxml.etree import SubElement

from xapiparser.exception import ParseError
from xapiparser.exception import UnsupportedError

COMMANDS = ('xCommand', 'xStatus', 'xConfiguration')
UNSUPPORTED = ('xGetxml', 'xEvent')
SSH_ONLY = ('Systemtools', 'Log', 'xPreferences', 'xFeedback', 'Echo')
INDEXED_TAG = r"\[(\d+)\]$"


def _rejoin(expr):
    return " ".join(expr)


def parse(expression):
    return _XApiParser.parse(expression)


class _XApiParser:

    @staticmethod
    def parse(expression, root=None, current=None):
        try:
            expr = shlex.split(expression)

            # base case
            if root is None:
                if expr[0].lower() in [u.lower() for u in SSH_ONLY]:
                    raise UnsupportedError('CLI expression is not supported in REST API')
                if expr[0].lower() in [u.lower() for u in UNSUPPORTED]:
                    raise UnsupportedError('Expression is not currently supported by xapiparser')
                if expr[0].lower() not in [c.lower() for c in COMMANDS]:
                    raise ParseError(f"Unknown command: {expr[0]}")
                root = Element(expr[0][1:])
                return _XApiParser.parse(_rejoin(expr[1:]), root=root, current=root)

            # exit case: end of expression reached
            elif not expr:
                return root

            # case: sub-element with text
            elif expr[0][-1] == ":":
                # [indexed] tag with value and trailing colon
                if re.search(INDEXED_TAG, expr[0][:-1]):
                    tag = expr[0][:-1].split("[")[0]
                    child = SubElement(current, tag)
                    val = re.search(INDEXED_TAG, expr[0][:-1]).group(1)
                    child.set("item", val)
                    child.text = expr[1]
                    return _XApiParser.parse(_rejoin(expr[2:]), root=root, current=current)
                # only trailing colon
                else:
                    child = SubElement(current, expr[0][:-1])
                    child.text = expr[1]
                    return _XApiParser.parse(_rejoin(expr[2:]), root=root, current=current)

            # case: [indexed] tag with value
            elif re.search(INDEXED_TAG, expr[0]):
                tag = expr[0].split("[")[0]
                child = SubElement(current, tag)
                val = re.search(INDEXED_TAG, expr[0]).group(1)
                child.set("item", val)
                child.text = expr[1]
                return _XApiParser.parse(_rejoin(expr[2:]), root=root, current=child)

            # case: index as next element and subsequent values are provided
            elif expr[1].isdigit() and expr[2]:
                child = SubElement(current, expr[0])
                child.set("item", expr[1])
                return _XApiParser.parse(_rejoin(expr[2:]), root=root, current=child)

            # case: basic child element
            else:
                child = SubElement(current, expr[0])
                return _XApiParser.parse(_rejoin(expr[1:]), root=root, current=child)

        except Exception:
            raise ParseError("Parsing failed")



import lxml.etree as etree
import pytest

from xapiparser import parse
from xapiparser.cli import main


@pytest.fixture
def strip_whitespace():
    return etree.XMLParser(remove_blank_text=True)


def test_main(strip_whitespace):
    main(['xCommand Audio Volume Set Level: 50'])


def test_basic(strip_whitespace):
    expression = 'xCommand Audio Volume Set Level: 50'
    xapi = """
    <Command>
        <Audio>
            <Volume>
                <Set>
                    <Level>50</Level>
                </Set>
            </Volume>
        </Audio>
    </Command>
    """
    expected = etree.XML(xapi, parser=strip_whitespace)
    parsed = parse(expression)
    assert etree.tostring(expected) == etree.tostring(parsed)


def test_item_attribute(strip_whitespace):
    expression = "xConfiguration Video Input Connector 2 InputSourceType: camera"
    xapi = """
    <Configuration>
        <Video>
            <Input>
                <Connector item="2">
                    <InputSourceType>camera</InputSourceType>
                </Connector>
            </Input>
        </Video>
    </Configuration>
    """
    expected = etree.XML(xapi, parser=strip_whitespace)
    parsed = parse(expression)
    assert etree.tostring(expected) == etree.tostring(parsed)


def test_key_value(strip_whitespace):
    expression = 'xCommand Dial Number: "12345" Protocol: H323'
    xapi = """
    <Command>
        <Dial>
            <Number>12345</Number>
            <Protocol>H323</Protocol>
        </Dial>
    </Command>
    """
    expected = etree.XML(xapi, parser=strip_whitespace)
    parsed = parse(expression)
    assert etree.tostring(expected) == etree.tostring(parsed)


def test_attribute_suffix_with_value(strip_whitespace):
    expression = 'xCommand HttpFeedback Register FeedbackSlot: 2 ServerUrl: https://status.oops.com/ Format: JSON ' \
                 'Expression[1]: /Event/UserInterface/Message/Prompt/Response ' \
                 'Expression[2]: /Event/UserInterface/Message/TextInput/Response/text ' \
                 'Expression[3]: /Event/CallDisconnect'
    xapi = """
    <Command>
        <HttpFeedback>
            <Register>
                <FeedbackSlot>2</FeedbackSlot>
                <ServerUrl>https://status.oops.com/</ServerUrl>
                <Format>JSON</Format>
                <Expression item="1">/Event/UserInterface/Message/Prompt/Response</Expression>
                <Expression item="2">/Event/UserInterface/Message/TextInput/Response/text</Expression>
                <Expression item="3">/Event/CallDisconnect</Expression>
            </Register>
        </HttpFeedback>
    </Command>
    """
    expected = etree.XML(xapi, parser=strip_whitespace)
    parsed = parse(expression)
    assert etree.tostring(expected) == etree.tostring(parsed)
